@@ -15,7 +15,7 @@
  *    except according to the terms contained in the LICENSE file.
  */
 
-const char libeinfo_copyright[] = "Copyright (c) 2007-2008 Roy Marples";
+static const char libeinfo_copyright[] = "Copyright (c) 2007-2008 Roy Marples";
 
 #include <sys/types.h>
 #include <sys/ioctl.h>
@@ -39,34 +39,6 @@ const char libeinfo_copyright[] = "Copyright (c) 2007-2008 Roy Marples";
 
 #include "einfo.h"
 #include "helpers.h"
-#include "hidden-visibility.h"
-
-hidden_proto(ecolor)
-hidden_proto(ebegin)
-hidden_proto(ebeginv)
-hidden_proto(ebracket)
-hidden_proto(eend)
-hidden_proto(eendv)
-hidden_proto(eerror)
-hidden_proto(eerrorn)
-hidden_proto(eerrorx)
-hidden_proto(eindent)
-hidden_proto(eindentv)
-hidden_proto(einfo)
-hidden_proto(einfon)
-hidden_proto(einfov)
-hidden_proto(einfovn)
-hidden_proto(elog)
-hidden_proto(eoutdent)
-hidden_proto(eoutdentv)
-hidden_proto(eprefix)
-hidden_proto(ewarn)
-hidden_proto(ewarnn)
-hidden_proto(ewarnv)
-hidden_proto(ewarnvn)
-hidden_proto(ewarnx)
-hidden_proto(ewend)
-hidden_proto(ewendv)
 
 /* Incase we cannot work out how many columns from ioctl, supply a default */
 #define DEFAULT_COLS		 80
@@ -510,7 +482,6 @@ eprefix(const char *EINFO_RESTRICT prefix)
 {
 	_eprefix = prefix;
 }
-hidden_def(eprefix)
 
 static void EINFO_PRINTF(2, 0)
 elogv(int level, const char *EINFO_RESTRICT fmt, va_list ap)
@@ -537,7 +508,6 @@ elog(int level, const char *EINFO_RESTRICT fmt, ...)
 	elogv(level, fmt, ap);
 	va_end(ap);
 }
-hidden_def(elog)
 
 static int
 _eindent(FILE * EINFO_RESTRICT stream)
@@ -576,7 +546,6 @@ _ecolor(FILE * EINFO_RESTRICT f, ECOLOR color)
 			return ecolors_str[i];
 	return "";
 }
-hidden_def(ecolor)
 
 const char *
 ecolor(ECOLOR color)
@@ -643,7 +612,6 @@ einfon(const char *EINFO_RESTRICT fmt, ...)
 	LASTCMD("einfon");
 	return retval;
 }
-hidden_def(einfon)
 
 int
 ewarnn(const char *EINFO_RESTRICT fmt, ...)
@@ -659,7 +627,6 @@ ewarnn(const char *EINFO_RESTRICT fmt, ...)
 	LASTCMD("ewarnn");
 	return retval;
 }
-hidden_def(ewarnn)
 
 int
 eerrorn(const char *EINFO_RESTRICT fmt, ...)
@@ -675,7 +642,6 @@ eerrorn(const char *EINFO_RESTRICT fmt, ...)
 	LASTCMD("errorn");
 	return retval;
 }
-hidden_def(eerrorn)
 
 int
 einfo(const char *EINFO_RESTRICT fmt, ...)
@@ -692,7 +658,6 @@ einfo(const char *EINFO_RESTRICT fmt, ...)
 	LASTCMD("einfo");
 	return retval;
 }
-hidden_def(einfo)
 
 int
 ewarn(const char *EINFO_RESTRICT fmt, ...)
@@ -710,7 +675,6 @@ ewarn(const char *EINFO_RESTRICT fmt, ...)
 	LASTCMD("ewarn");
 	return retval;
 }
-hidden_def(ewarn)
 
 void
 ewarnx(const char *EINFO_RESTRICT fmt, ...)
@@ -727,7 +691,6 @@ ewarnx(const char *EINFO_RESTRICT fmt, ...)
 	}
 	exit(EXIT_FAILURE);
 }
-hidden_def(ewarnx)
 
 int
 eerror(const char *EINFO_RESTRICT fmt, ...)
@@ -745,7 +708,6 @@ eerror(const char *EINFO_RESTRICT fmt, ...)
 	LASTCMD("eerror");
 	return retval;
 }
-hidden_def(eerror)
 
 void
 eerrorx(const char *EINFO_RESTRICT fmt, ...)
@@ -761,7 +723,6 @@ eerrorx(const char *EINFO_RESTRICT fmt, ...)
 	}
 	exit(EXIT_FAILURE);
 }
-hidden_def(eerrorx)
 
 int
 ebegin(const char *EINFO_RESTRICT fmt, ...)
@@ -780,7 +741,6 @@ ebegin(const char *EINFO_RESTRICT fmt, ...)
 	LASTCMD("ebegin");
 	return retval;
 }
-hidden_def(ebegin)
 
 static void
 _eend(FILE * EINFO_RESTRICT fp, int col, ECOLOR color, const char *msg)
@@ -854,7 +814,6 @@ eend(int retval, const char *EINFO_RESTRICT fmt, ...)
 	LASTCMD("eend");
 	return retval;
 }
-hidden_def(eend)
 
 int
 ewend(int retval, const char *EINFO_RESTRICT fmt, ...)
@@ -869,21 +828,19 @@ ewend(int retval, const char *EINFO_RESTRICT fmt, ...)
 	LASTCMD("ewend");
 	return retval;
 }
-hidden_def(ewend)
 
 void
 ebracket(int col, ECOLOR color, const char *msg)
 {
 	_eend(stdout, col, color, msg);
 }
-hidden_def(ebracket)
 
 void
 eindent(void)
 {
 	char *env = getenv("EINFO_INDENT");
 	int amount = 0;
-	char num[10];
+	char *num;
 
 	if (env) {
 		errno = 0;
@@ -894,16 +851,16 @@ eindent(void)
 	amount += INDENT_WIDTH;
 	if (amount > INDENT_MAX)
 		amount = INDENT_MAX;
-	snprintf(num, 10, "%08d", amount);
+	xasprintf(&num, "%08d", amount);
 	setenv("EINFO_INDENT", num, 1);
+	free(num);
 }
-hidden_def(eindent)
 
 void eoutdent(void)
 {
 	char *env = getenv("EINFO_INDENT");
 	int amount = 0;
-	char num[10];
+	char *num = NULL;
 	int serrno = errno;
 
 	if (!env)
@@ -917,12 +874,12 @@ void eoutdent(void)
 	if (amount <= 0)
 		unsetenv("EINFO_INDENT");
 	else {
-		snprintf(num, 10, "%08d", amount);
+		xasprintf(&num, "%08d", amount);
 		setenv("EINFO_INDENT", num, 1);
+		free(num);
 	}
 	errno = serrno;
 }
-hidden_def(eoutdent)
 
 int
 einfovn(const char *EINFO_RESTRICT fmt, ...)
@@ -938,7 +895,6 @@ einfovn(const char *EINFO_RESTRICT fmt, ...)
 	LASTCMD("einfovn");
 	return retval;
 }
-hidden_def(einfovn)
 
 int
 ewarnvn(const char *EINFO_RESTRICT fmt, ...)
@@ -954,7 +910,6 @@ ewarnvn(const char *EINFO_RESTRICT fmt, ...)
 	LASTCMD("ewarnvn");
 	return retval;
 }
-hidden_def(ewarnvn)
 
 int
 einfov(const char *EINFO_RESTRICT fmt, ...)
@@ -971,7 +926,6 @@ einfov(const char *EINFO_RESTRICT fmt, ...)
 	LASTCMD("einfov");
 	return retval;
 }
-hidden_def(einfov)
 
 int
 ewarnv(const char *EINFO_RESTRICT fmt, ...)
@@ -988,7 +942,6 @@ ewarnv(const char *EINFO_RESTRICT fmt, ...)
 	LASTCMD("ewarnv");
 	return retval;
 }
-hidden_def(ewarnv)
 
 int
 ebeginv(const char *EINFO_RESTRICT fmt, ...)
@@ -1008,7 +961,6 @@ ebeginv(const char *EINFO_RESTRICT fmt, ...)
 	LASTCMD("ebeginv");
 	return retval;
 }
-hidden_def(ebeginv)
 
 int
 eendv(int retval, const char *EINFO_RESTRICT fmt, ...)
@@ -1023,7 +975,6 @@ eendv(int retval, const char *EINFO_RESTRICT fmt, ...)
 	LASTCMD("eendv");
 	return retval;
 }
-hidden_def(eendv)
 
 int
 ewendv(int retval, const char *EINFO_RESTRICT fmt, ...)
@@ -1038,7 +989,6 @@ ewendv(int retval, const char *EINFO_RESTRICT fmt, ...)
 	LASTCMD("ewendv");
 	return retval;
 }
-hidden_def(ewendv)
 
 void
 eindentv(void)
@@ -1046,7 +996,6 @@ eindentv(void)
 	if (is_verbose())
 		eindent();
 }
-hidden_def(eindentv)
 
 void
 eoutdentv(void)
@@ -1054,4 +1003,3 @@ eoutdentv(void)
 	if (is_verbose())
 		eoutdent();
 }
-hidden_def(eoutdentv)
