@@ -27,97 +27,185 @@
 #include "rc-misc.h"
 #include "_usage.h"
 
+enum format_t {
+	FORMAT_DEFAULT,
+	FORMAT_INI,
+};
+
 const char *applet = NULL;
 const char *extraopts = NULL;
-const char *getoptstring = "aclmrsu" getoptstring_COMMON;
+const char *getoptstring = "acf:lmrsSu" getoptstring_COMMON;
 const struct option longopts[] = {
 	{"all",         0, NULL, 'a'},
 	{"crashed",     0, NULL, 'c'},
+	{"format",     1, NULL, 'f'},
 	{"list",        0, NULL, 'l'},
 	{"manual",        0, NULL, 'm'},
 	{"runlevel",    0, NULL, 'r'},
 	{"servicelist", 0, NULL, 's'},
+	{"supervised", 0, NULL, 'S'},
 	{"unused",      0, NULL, 'u'},
 	longopts_COMMON
 };
 const char * const longopts_help[] = {
 	"Show services from all run levels",
 	"Show crashed services",
+	"format status to be parsable (currently arg must be ini)",
 	"Show list of run levels",
 	"Show manually started services",
 	"Show the name of the current runlevel",
 	"Show service list",
+	"show supervised services",
 	"Show services not assigned to any runlevel",
 	longopts_help_COMMON
 };
 const char *usagestring = ""						\
-	"Usage: rc-status [options] <runlevel>...\n"		\
+	"Usage: rc-status [options] -f ini <runlevel>...\n"		\
 	"   or: rc-status [options] [-a | -c | -l | -m | -r | -s | -u]";
 
-static bool test_crashed = false;
 static RC_DEPTREE *deptree;
 static RC_STRINGLIST *types;
 
 static RC_STRINGLIST *levels, *services, *tmp, *alist;
 static RC_STRINGLIST *sservices, *nservices, *needsme;
 
-static void
-print_level(const char *prefix, const char *level)
+static void print_level(const char *prefix, const char *level,
+		enum format_t format)
 {
-	if (prefix)
-		printf("%s ", prefix);
-	printf ("Runlevel: ");
-	if (isatty(fileno(stdout)))
-		printf("%s%s%s\n",
-		       ecolor(ECOLOR_HILITE),
-		       level,
-		       ecolor(ECOLOR_NORMAL));
-	else
-		printf("%s\n", level);
+	switch (format) {
+	case FORMAT_DEFAULT:
+		if (prefix)
+			printf("%s ", prefix);
+		printf ("Runlevel: ");
+		if (isatty(fileno(stdout)))
+			printf("%s%s%s\n",
+					ecolor(ECOLOR_HILITE), level, ecolor(ECOLOR_NORMAL));
+		else
+			printf("%s\n", level);
+		break;
+	case FORMAT_INI:
+		printf("%s", "[");
+		if (prefix)
+			printf("%s ", prefix);
+		printf("%s]\n", level);
+		break;
+	}
 }
 
-static void
-print_service(const char *service)
+static char *get_uptime(const char *service)
 {
-	char status[10];
-	int cols =  printf(" %s", service);
+	RC_SERVICE state = rc_service_state(service);
+	char *start_count;
+	time_t now;
+	char *start_time_string;
+	time_t start_time;
+	time_t time_diff;
+	time_t diff_days = (time_t) 0;
+	time_t diff_hours = (time_t) 0;
+	time_t diff_mins = (time_t) 0;
+	time_t diff_secs = (time_t) 0;
+	char *uptime = NULL;
+
+	if (state & RC_SERVICE_STARTED) {
+		start_count = rc_service_value_get(service, "start_count");
+		start_time_string = rc_service_value_get(service, "start_time");
+		if (start_count && start_time_string) {
+			start_time = to_time_t(start_time_string);
+			now = time(NULL);
+			time_diff = (time_t) difftime(now, start_time);
+			diff_secs = time_diff;
+			if (diff_secs > (time_t) 86400) {
+				diff_days = diff_secs / (time_t) 86400;
+				diff_secs %= diff_days * (time_t) 86400;
+			}
+			if (diff_secs > (time_t) 3600) {
+				diff_hours = diff_secs / (time_t) 3600;
+				diff_secs %= diff_hours * (time_t) 3600;
+			}
+			if (diff_secs > (time_t) 60) {
+				diff_mins = diff_secs / (time_t) 60;
+				diff_secs %= diff_mins * (time_t) 60;
+			}
+			if (diff_days > 0)
+				xasprintf(&uptime,
+						"%ld day(s) %02ld:%02ld:%02ld (%s)",
+						diff_days, diff_hours, diff_mins, diff_secs,
+						start_count);
+			else
+				xasprintf(&uptime,
+						"%02ld:%02ld:%02ld (%s)",
+						diff_hours, diff_mins, diff_secs, start_count);
+		}
+	}
+	return uptime;
+}
+
+static void print_service(const char *service, enum format_t format)
+{
+	char *status = NULL;
+	char *uptime = NULL;
+	char *child_pid = NULL;
+	char *start_time = NULL;
+	int cols;
 	const char *c = ecolor(ECOLOR_GOOD);
 	RC_SERVICE state = rc_service_state(service);
 	ECOLOR color = ECOLOR_BAD;
 
 	if (state & RC_SERVICE_STOPPING)
-		snprintf(status, sizeof(status), "stopping ");
+		xasprintf(&status, "stopping ");
 	else if (state & RC_SERVICE_STARTING) {
-		snprintf(status, sizeof(status), "starting ");
+		xasprintf(&status, "starting ");
 		color = ECOLOR_WARN;
 	} else if (state & RC_SERVICE_INACTIVE) {
-		snprintf(status, sizeof(status), "inactive ");
+		xasprintf(&status, "inactive ");
 		color = ECOLOR_WARN;
 	} else if (state & RC_SERVICE_STARTED) {
 		errno = 0;
-		if (test_crashed &&
-		    rc_service_daemons_crashed(service) &&
-		    errno != EACCES)
+		if (rc_service_daemons_crashed(service) && errno != EACCES)
 		{
-			snprintf(status, sizeof(status), " crashed ");
+			child_pid = rc_service_value_get(service, "child_pid");
+			start_time = rc_service_value_get(service, "start_time");
+			if (start_time && child_pid)
+				xasprintf(&status, " unsupervised ");
+			else
+				xasprintf(&status, " crashed ");
+			free(child_pid);
+			free(start_time);
 		} else {
-			snprintf(status, sizeof(status), " started ");
+			uptime = get_uptime(service);
+			if (uptime) {
+				xasprintf(&status, " started %s", uptime);
+				free(uptime);
+			} else
+				xasprintf(&status, " started ");
 			color = ECOLOR_GOOD;
 		}
 	} else if (state & RC_SERVICE_SCHEDULED) {
-		snprintf(status, sizeof(status), "scheduled");
+		xasprintf(&status, "scheduled");
+		color = ECOLOR_WARN;
+	} else if (state & RC_SERVICE_FAILED) {
+		xasprintf(&status, "failed");
 		color = ECOLOR_WARN;
 	} else
-		snprintf(status, sizeof(status), " stopped ");
+		xasprintf(&status, " stopped ");
 
 	errno = 0;
-	if (c && *c && isatty(fileno(stdout)))
-		printf("\n");
-	ebracket(cols, color, status);
+	switch (format) {
+	case FORMAT_DEFAULT:
+		cols =  printf(" %s", service);
+		if (c && *c && isatty(fileno(stdout)))
+			printf("\n");
+		ebracket(cols, color, status);
+		break;
+	case FORMAT_INI:
+		printf("%s = %s\n", service, status);
+		break;
+	}
+	free(status);
 }
 
-static void
-print_services(const char *runlevel, RC_STRINGLIST *svcs)
+static void print_services(const char *runlevel, RC_STRINGLIST *svcs,
+		enum format_t format)
 {
 	RC_STRINGLIST *l = NULL;
 	RC_STRING *s;
@@ -131,7 +219,7 @@ print_services(const char *runlevel, RC_STRINGLIST *svcs)
 		TAILQ_FOREACH(s, svcs, entries)
 			if (!runlevel ||
 			    rc_service_in_runlevel(s->value, runlevel))
-				print_service(s->value);
+				print_service(s->value, format);
 		return;
 	}
 	if (!types) {
@@ -151,13 +239,12 @@ print_services(const char *runlevel, RC_STRINGLIST *svcs)
 		if (!rc_stringlist_find(svcs, s->value))
 			continue;
 		if (!runlevel || rc_service_in_runlevel(s->value, runlevel))
-			print_service(s->value);
+			print_service(s->value, format);
 	}
 	rc_stringlist_free(l);
 }
 
-static void
-print_stacked_services(const char *runlevel)
+static void print_stacked_services(const char *runlevel, enum format_t format)
 {
 	RC_STRINGLIST *stackedlevels, *servicelist;
 	RC_STRING *stackedlevel;
@@ -166,9 +253,9 @@ print_stacked_services(const char *runlevel)
 	TAILQ_FOREACH(stackedlevel, stackedlevels, entries) {
 		if (rc_stringlist_find(levels, stackedlevel->value) != NULL)
 			continue;
-		print_level("Stacked", stackedlevel->value);
+		print_level("Stacked", stackedlevel->value, format);
 		servicelist = rc_services_in_runlevel(stackedlevel->value);
-		print_services(stackedlevel->value, servicelist);
+		print_services(stackedlevel->value, servicelist, format);
 		rc_stringlist_free(servicelist);
 	}
 	rc_stringlist_free(stackedlevels);
@@ -177,13 +264,14 @@ print_stacked_services(const char *runlevel)
 
 int main(int argc, char **argv)
 {
-    RC_STRING *s, *l, *t, *level;
+	RC_SERVICE state;
+	RC_STRING *s, *l, *t, *level;
+	enum format_t format = FORMAT_DEFAULT;
+	bool levels_given = false;
 	bool show_all = false;
 	char *p, *runlevel = NULL;
 	int opt, retval = 0;
 
-	test_crashed = _rc_can_find_pids();
-
 	applet = basename_c(argv[0]);
 	while ((opt = getopt_long(argc, argv, getoptstring, longopts,
 				  (int *) 0)) != -1)
@@ -202,6 +290,12 @@ int main(int argc, char **argv)
 				}
 			goto exit;
 			/* NOTREACHED */
+		case 'f':
+			if (strcasecmp(optarg, "ini") == 0)
+				format = FORMAT_INI;
+			else
+				eerrorx("%s: invalid argument to --format switch\n", applet);
+			break;
 		case 'l':
 			levels = rc_runlevel_list();
 			TAILQ_FOREACH(l, levels, entries)
@@ -226,16 +320,24 @@ int main(int argc, char **argv)
 					free(s->value);
 					free(s);
 				}
-			print_services(NULL, services);
+			print_services(NULL, services, FORMAT_DEFAULT);
 			goto exit;
 		case 'r':
 			runlevel = rc_runlevel_get();
 			printf("%s\n", runlevel);
 			goto exit;
 			/* NOTREACHED */
+		case 'S':
+			services = rc_services_in_state(RC_SERVICE_STARTED);
+			TAILQ_FOREACH_SAFE(s, services, entries, t)
+				if (!rc_service_value_get(s->value, "child_pid"))
+					TAILQ_REMOVE(services, s, entries);
+			print_services(NULL, services, FORMAT_DEFAULT);
+			goto exit;
+			/* NOTREACHED */
 		case 's':
 			services = rc_services_in_runlevel(NULL);
-			print_services(NULL, services);
+			print_services(NULL, services, FORMAT_DEFAULT);
 			goto exit;
 			/* NOTREACHED */
 		case 'u':
@@ -250,7 +352,7 @@ int main(int argc, char **argv)
 						break;
 					}
 			}
-			print_services(NULL, services);
+			print_services(NULL, services, FORMAT_DEFAULT);
 			goto exit;
 			/* NOTREACHED */
 
@@ -262,6 +364,7 @@ int main(int argc, char **argv)
 	opt = (optind < argc) ? 0 : 1;
 	while (optind < argc) {
 		if (rc_runlevel_exists(argv[optind])) {
+			levels_given = true;
 			rc_stringlist_add(levels, argv[optind++]);
 			opt++;
 		} else
@@ -278,21 +381,21 @@ int main(int argc, char **argv)
 	deptree = _rc_deptree_load(0, NULL);
 
 	TAILQ_FOREACH(l, levels, entries) {
-		print_level(NULL, l->value);
+		print_level(NULL, l->value, format);
 		services = rc_services_in_runlevel(l->value);
-		print_services(l->value, services);
-		print_stacked_services(l->value);
+		print_services(l->value, services, format);
+		print_stacked_services(l->value, format);
 		rc_stringlist_free(nservices);
 		nservices = NULL;
 		rc_stringlist_free(services);
 		services = NULL;
 	}
 
-	if (show_all || argc < 2) {
+	if (show_all || !levels_given) {
 		/* Show hotplugged services */
-		print_level("Dynamic", "hotplugged");
+		print_level("Dynamic", "hotplugged", format);
 		services = rc_services_in_state(RC_SERVICE_HOTPLUGGED);
-		print_services(NULL, services);
+		print_services(NULL, services, format);
 		rc_stringlist_free(services);
 		services = NULL;
 
@@ -314,11 +417,14 @@ int main(int argc, char **argv)
 			free(nservices);
 		}
 		TAILQ_FOREACH_SAFE(s, services, entries, t) {
+			state = rc_service_state(s->value);
 			if ((rc_stringlist_find(sservices, s->value) ||
-			    (rc_service_state(s->value) & ( RC_SERVICE_STOPPED | RC_SERVICE_HOTPLUGGED)))) {
-				TAILQ_REMOVE(services, s, entries);
-				free(s->value);
-				free(s);
+			    (state & ( RC_SERVICE_STOPPED | RC_SERVICE_HOTPLUGGED)))) {
+				if (! (state & RC_SERVICE_FAILED)) {
+					TAILQ_REMOVE(services, s, entries);
+					free(s->value);
+					free(s);
+				}
 			}
 		}
 		needsme = rc_stringlist_new();
@@ -346,10 +452,10 @@ int main(int argc, char **argv)
 		 * be added to the list
 		 */
 		unsetenv("RC_SVCNAME");
-		print_level("Dynamic", "needed/wanted");
-		print_services(NULL, nservices);
-		print_level("Dynamic", "manual");
-		print_services(NULL, services);
+		print_level("Dynamic", "needed/wanted", format);
+		print_services(NULL, nservices, format);
+		print_level("Dynamic", "manual", format);
+		print_services(NULL, services, format);
 	}
 
 exit:
