@@ -19,10 +19,6 @@
  *    except according to the terms contained in the LICENSE file.
  */
 
-/* nano seconds */
-#define POLL_INTERVAL   20000000
-#define WAIT_PIDFILE   500000000
-#define ONE_SECOND    1000000000
 #define ONE_MS           1000000
 
 #include <sys/types.h>
@@ -63,11 +59,14 @@ static struct pam_conv conv = { NULL, NULL};
 #include "queue.h"
 #include "rc.h"
 #include "rc-misc.h"
+#include "rc-pipes.h"
+#include "rc-schedules.h"
 #include "_usage.h"
+#include "helpers.h"
 
 const char *applet = NULL;
 const char *extraopts = NULL;
-const char *getoptstring = "I:KN:PR:Sa:bc:d:e:g:ik:mn:op:s:tu:r:w:x:1:2:" \
+const char *getoptstring = "I:KN:PR:Sa:bc:d:e:g:ik:mn:op:s:tu:r:w:x:1:2:3:4:" \
 	getoptstring_COMMON;
 const struct option longopts[] = {
 	{ "ionice",       1, NULL, 'I'},
@@ -95,6 +94,8 @@ const struct option longopts[] = {
 	{ "exec",         1, NULL, 'x'},
 	{ "stdout",       1, NULL, '1'},
 	{ "stderr",       1, NULL, '2'},
+	{ "stdout-logger",1, NULL, '3'},
+	{ "stderr-logger",1, NULL, '4'},
 	{ "progress",     0, NULL, 'P'},
 	longopts_COMMON
 };
@@ -124,25 +125,13 @@ const char * const longopts_help[] = {
 	"Binary to start/stop",
 	"Redirect stdout to file",
 	"Redirect stderr to file",
+	"Redirect stdout to process",
+	"Redirect stderr to process",
 	"Print dots each second while waiting",
 	longopts_help_COMMON
 };
 const char *usagestring = NULL;
 
-typedef struct scheduleitem
-{
-	enum
-		{
-			SC_TIMEOUT,
-			SC_SIGNAL,
-			SC_GOTO,
-			SC_FOREVER
-		} type;
-	int value;
-	struct scheduleitem *gotoitem;
-	TAILQ_ENTRY(scheduleitem) entries;
-} SCHEDULEITEM;
-TAILQ_HEAD(, scheduleitem) schedule;
 static char **nav;
 
 static char *changeuser, *ch_root, *ch_dir;
@@ -153,7 +142,9 @@ extern char **environ;
 # define SYS_ioprio_set __NR_ioprio_set
 #endif
 #if !defined(__DragonFly__)
-static inline int ioprio_set(int which, int who, int ioprio)
+static inline int ioprio_set(int which _unused,
+			     int who _unused,
+			     int ioprio _unused)
 {
 #ifdef SYS_ioprio_set
 	return syscall(SYS_ioprio_set, which, who, ioprio);
@@ -163,20 +154,6 @@ static inline int ioprio_set(int which, int who, int ioprio)
 }
 #endif
 
-static void
-free_schedulelist(void)
-{
-	SCHEDULEITEM *s1 = TAILQ_FIRST(&schedule);
-	SCHEDULEITEM *s2;
-
-	while (s1) {
-		s2 = TAILQ_NEXT(s1, entries);
-		free(s1);
-		s1 = s2;
-	}
-	TAILQ_INIT(&schedule);
-}
-
 static void
 cleanup(void)
 {
@@ -185,385 +162,25 @@ cleanup(void)
 	free_schedulelist();
 }
 
-static int
-parse_signal(const char *sig)
-{
-	typedef struct signalpair
-	{
-		const char *name;
-		int signal;
-	} SIGNALPAIR;
-
-	static const SIGNALPAIR signallist[] = {
-		{ "ABRT",	SIGABRT	},
-		{ "ALRM",	SIGALRM	},
-		{ "FPE",	SIGFPE	},
-		{ "HUP",	SIGHUP	},
-		{ "ILL",	SIGILL	},
-		{ "INT",	SIGINT	},
-		{ "KILL",	SIGKILL	},
-		{ "PIPE",	SIGPIPE	},
-		{ "QUIT",	SIGQUIT	},
-		{ "SEGV",	SIGSEGV	},
-		{ "TERM",	SIGTERM	},
-		{ "USR1",	SIGUSR1	},
-		{ "USR2",	SIGUSR2	},
-		{ "CHLD",	SIGCHLD	},
-		{ "CONT",	SIGCONT	},
-		{ "STOP",	SIGSTOP	},
-		{ "TSTP",	SIGTSTP	},
-		{ "TTIN",	SIGTTIN	},
-		{ "TTOU",	SIGTTOU	},
-		{ "NULL",	0 },
-	};
-
-	unsigned int i = 0;
-	const char *s;
-
-	if (!sig || *sig == '\0')
-		return -1;
-
-	if (sscanf(sig, "%u", &i) == 1) {
-		if (i < NSIG)
-			return i;
-		eerrorx("%s: `%s' is not a valid signal", applet, sig);
-	}
-
-	if (strncmp(sig, "SIG", 3) == 0)
-		s = sig + 3;
-	else
-		s = NULL;
-
-	for (i = 0; i < ARRAY_SIZE(signallist); ++i)
-		if (strcmp(sig, signallist[i].name) == 0 ||
-		    (s && strcmp(s, signallist[i].name) == 0))
-			return signallist[i].signal;
-
-	eerrorx("%s: `%s' is not a valid signal", applet, sig);
-	/* NOTREACHED */
-}
-
-static SCHEDULEITEM *
-parse_schedule_item(const char *string)
-{
-	const char *after_hyph;
-	int sig;
-	SCHEDULEITEM *item = xmalloc(sizeof(*item));
-
-	item->value = 0;
-	item->gotoitem = NULL;
-	if (strcmp(string,"forever") == 0)
-		item->type = SC_FOREVER;
-	else if (isdigit((unsigned char)string[0])) {
-		item->type = SC_TIMEOUT;
-		errno = 0;
-		if (sscanf(string, "%d", &item->value) != 1)
-			eerrorx("%s: invalid timeout value in schedule `%s'",
-			    applet, string);
-	} else if ((after_hyph = string + (string[0] == '-')) &&
-	    ((sig = parse_signal(after_hyph)) != -1))
-	{
-		item->type = SC_SIGNAL;
-		item->value = (int)sig;
-	} else
-		eerrorx("%s: invalid schedule item `%s'", applet, string);
-
-	return item;
-}
-
-static void
-parse_schedule(const char *string, int timeout)
-{
-	char buffer[20];
-	const char *slash;
-	int count = 0;
-	SCHEDULEITEM *repeatat = NULL;
-	size_t len;
-	SCHEDULEITEM *item;
-
-	if (string)
-		for (slash = string; *slash; slash++)
-			if (*slash == '/')
-				count++;
-
-	free_schedulelist();
-
-	if (count == 0) {
-		item = xmalloc(sizeof(*item));
-		item->type = SC_SIGNAL;
-		item->value = timeout;
-		item->gotoitem = NULL;
-		TAILQ_INSERT_TAIL(&schedule, item, entries);
-
-		item = xmalloc(sizeof(*item));
-		item->type = SC_TIMEOUT;
-		item->gotoitem = NULL;
-		TAILQ_INSERT_TAIL(&schedule, item, entries);
-		if (string) {
-			if (sscanf(string, "%d", &item->value) != 1)
-				eerrorx("%s: invalid timeout in schedule",
-				    applet);
-		} else
-			item->value = 5;
-
-		return;
-	}
-
-	while (string != NULL) {
-		if ((slash = strchr(string, '/')))
-			len = slash - string;
-		else
-			len = strlen(string);
-
-		if (len >= (ptrdiff_t)sizeof(buffer))
-			eerrorx("%s: invalid schedule item, far too long",
-			    applet);
-
-		memcpy(buffer, string, len);
-		buffer[len] = 0;
-		string = slash ? slash + 1 : NULL;
-
-		item = parse_schedule_item(buffer);
-		TAILQ_INSERT_TAIL(&schedule, item, entries);
-		if (item->type == SC_FOREVER) {
-			if (repeatat)
-				eerrorx("%s: invalid schedule, `forever' "
-				    "appears more than once", applet);
-
-			repeatat = item;
-			continue;
-		}
-	}
-
-	if (repeatat) {
-		item = xmalloc(sizeof(*item));
-		item->type = SC_GOTO;
-		item->value = 0;
-		item->gotoitem = repeatat;
-		TAILQ_INSERT_TAIL(&schedule, item, entries);
-	}
-
-	return;
-}
-
-static pid_t
-get_pid(const char *pidfile)
-{
-	FILE *fp;
-	pid_t pid;
-
-	if (! pidfile)
-		return -1;
-
-	if ((fp = fopen(pidfile, "r")) == NULL) {
-		ewarnv("%s: fopen `%s': %s", applet, pidfile, strerror(errno));
-		return -1;
-	}
-
-	if (fscanf(fp, "%d", &pid) != 1) {
-		ewarnv("%s: no pid found in `%s'", applet, pidfile);
-		fclose(fp);
-		return -1;
-	}
-
-	fclose(fp);
-
-	return pid;
-}
-
-/* return number of processed killed, -1 on error */
-static int
-do_stop(const char *exec, const char *const *argv,
-    pid_t pid, uid_t uid,int sig, bool test)
-{
-	RC_PIDLIST *pids;
-	RC_PID *pi;
-	RC_PID *np;
-	bool killed;
-	int nkilled = 0;
-
-	if (pid)
-		pids = rc_find_pids(NULL, NULL, 0, pid);
-	else
-		pids = rc_find_pids(exec, argv, uid, pid);
-
-	if (!pids)
-		return 0;
-
-	LIST_FOREACH_SAFE(pi, pids, entries, np) {
-		if (test) {
-			einfo("Would send signal %d to PID %d", sig, pi->pid);
-			nkilled++;
-		} else {
-			ebeginv("Sending signal %d to PID %d", sig, pi->pid);
-			errno = 0;
-			killed = (kill(pi->pid, sig) == 0 ||
-			    errno == ESRCH ? true : false);
-			eendv(killed ? 0 : 1,
-				"%s: failed to send signal %d to PID %d: %s",
-				applet, sig, pi->pid, strerror(errno));
-			if (!killed) {
-				nkilled = -1;
-			} else {
-				if (nkilled != -1)
-					nkilled++;
-			}
-		}
-		free(pi);
-	}
-
-	free(pids);
-	return nkilled;
-}
-
-static int
-run_stop_schedule(const char *exec, const char *const *argv,
-    const char *pidfile, uid_t uid,
-    bool test, bool progress)
-{
-	SCHEDULEITEM *item = TAILQ_FIRST(&schedule);
-	int nkilled = 0;
-	int tkilled = 0;
-	int nrunning = 0;
-	long nloops, nsecs;
-	struct timespec ts;
-	pid_t pid = 0;
-	const char *const *p;
-	bool progressed = false;
-
-	if (exec)
-		einfov("Will stop %s", exec);
-	if (pidfile)
-		einfov("Will stop PID in pidfile `%s'", pidfile);
-	if (uid)
-		einfov("Will stop processes owned by UID %d", uid);
-	if (argv && *argv) {
-		einfovn("Will stop processes of `");
-		if (rc_yesno(getenv("EINFO_VERBOSE"))) {
-			for (p = argv; p && *p; p++) {
-				if (p != argv)
-					printf(" ");
-				printf("%s", *p);
-			}
-			printf("'\n");
-		}
-	}
-
-	if (pidfile) {
-		pid = get_pid(pidfile);
-		if (pid == -1)
-			return 0;
-	}
-
-	while (item) {
-		switch (item->type) {
-		case SC_GOTO:
-			item = item->gotoitem;
-			continue;
-
-		case SC_SIGNAL:
-			nrunning = 0;
-			nkilled = do_stop(exec, argv, pid, uid, item->value, test);
-			if (nkilled == 0) {
-				if (tkilled == 0) {
-					if (progressed)
-						printf("\n");
-						eerror("%s: no matching processes found", applet);
-				}
-				return tkilled;
-			}
-			else if (nkilled == -1)
-				return 0;
-
-			tkilled += nkilled;
-			break;
-		case SC_TIMEOUT:
-			if (item->value < 1) {
-				item = NULL;
-				break;
-			}
-
-			ts.tv_sec = 0;
-			ts.tv_nsec = POLL_INTERVAL;
-
-			for (nsecs = 0; nsecs < item->value; nsecs++) {
-				for (nloops = 0;
-				     nloops < ONE_SECOND / POLL_INTERVAL;
-				     nloops++)
-				{
-					if ((nrunning = do_stop(exec, argv,
-						    pid, uid, 0, test)) == 0)
-						return 0;
-
-
-					if (nanosleep(&ts, NULL) == -1) {
-						if (progressed) {
-							printf("\n");
-							progressed = false;
-						}
-						if (errno == EINTR)
-							eerror("%s: caught an"
-							    " interrupt", applet);
-						else {
-							eerror("%s: nanosleep: %s",
-							    applet, strerror(errno));
-							return 0;
-						}
-					}
-				}
-				if (progress) {
-					printf(".");
-					fflush(stdout);
-					progressed = true;
-				}
-			}
-			break;
-		default:
-			if (progressed) {
-				printf("\n");
-				progressed = false;
-			}
-			eerror("%s: invalid schedule item `%d'",
-			    applet, item->type);
-			return 0;
-		}
-
-		if (item)
-			item = TAILQ_NEXT(item, entries);
-	}
-
-	if (test || (tkilled > 0 && nrunning == 0))
-		return nkilled;
-
-	if (progressed)
-		printf("\n");
-	if (nrunning == 1)
-		eerror("%s: %d process refused to stop", applet, nrunning);
-	else
-		eerror("%s: %d process(es) refused to stop", applet, nrunning);
-
-	return -nrunning;
-}
-
 static void
 handle_signal(int sig)
 {
 	int status;
 	int serrno = errno;
-	char signame[10] = { '\0' };
+	char *signame = NULL;
 
 	switch (sig) {
 	case SIGINT:
-		if (!signame[0])
-			snprintf(signame, sizeof(signame), "SIGINT");
+		if (!signame)
+			xasprintf(&signame, "SIGINT");
 		/* FALLTHROUGH */
 	case SIGTERM:
-		if (!signame[0])
-			snprintf(signame, sizeof(signame), "SIGTERM");
+		if (!signame)
+			xasprintf(&signame, "SIGTERM");
 		/* FALLTHROUGH */
 	case SIGQUIT:
-		if (!signame[0])
-			snprintf(signame, sizeof(signame), "SIGQUIT");
+		if (!signame)
+			xasprintf(&signame, "SIGQUIT");
 		eerrorx("%s: caught %s, aborting", applet, signame);
 		/* NOTREACHED */
 
@@ -582,6 +199,9 @@ handle_signal(int sig)
 		eerror("%s: caught unknown signal %d", applet, sig);
 	}
 
+	/* free signame */
+	free(signame);
+
 	/* Restore errno */
 	errno = serrno;
 }
@@ -590,7 +210,6 @@ static char *
 expand_home(const char *home, const char *path)
 {
 	char *opath, *ppath, *p, *nh;
-	size_t len;
 	struct passwd *pw;
 
 	if (!path || *path != '~')
@@ -621,9 +240,7 @@ expand_home(const char *home, const char *path)
 		return xstrdup(home);
 	}
 
-	len = strlen(ppath) + strlen(home) + 1;
-	nh = xmalloc(len);
-	snprintf(nh, len, "%s%s", home, ppath);
+	xasprintf(&nh, "%s%s", home, ppath);
 	free(opath);
 	return nh;
 }
@@ -642,6 +259,7 @@ int main(int argc, char **argv)
 #endif
 
 	int opt;
+	size_t size = 0;
 	bool start = false;
 	bool stop = false;
 	bool oknodo = false;
@@ -663,10 +281,13 @@ int main(int argc, char **argv)
 	int tid = 0;
 	char *redirect_stderr = NULL;
 	char *redirect_stdout = NULL;
+	char *stderr_process = NULL;
+	char *stdout_process = NULL;
 	int stdin_fd;
 	int stdout_fd;
 	int stderr_fd;
 	pid_t pid, spid;
+	RC_PIDLIST *pids;
 	int i;
 	char *svcname = getenv("RC_SVCNAME");
 	RC_STRINGLIST *env_list;
@@ -674,10 +295,10 @@ int main(int argc, char **argv)
 	char *tmp, *newpath, *np;
 	char *p;
 	char *token;
-	char exec_file[PATH_MAX];
+	char *exec_file = NULL;
 	struct passwd *pw;
 	struct group *gr;
-	char line[130];
+	char *line = NULL;
 	FILE *fp;
 	size_t len;
 	mode_t numask = 022;
@@ -685,7 +306,6 @@ int main(int argc, char **argv)
 	unsigned int start_wait = 0;
 
 	applet = basename_c(argv[0]);
-	TAILQ_INIT(&schedule);
 	atexit(cleanup);
 
 	signal_setup(SIGINT, handle_signal);
@@ -696,17 +316,17 @@ int main(int argc, char **argv)
 		if (sscanf(tmp, "%d", &nicelevel) != 1)
 			eerror("%s: invalid nice level `%s' (SSD_NICELEVEL)",
 			    applet, tmp);
-		if ((tmp = getenv("SSD_IONICELEVEL"))) {
-			int n = sscanf(tmp, "%d:%d", &ionicec, &ioniced);
-			if (n != 1 && n != 2)
-				eerror("%s: invalid ionice level `%s' (SSD_IONICELEVEL)",
-				    applet, tmp);
-			if (ionicec == 0)
-				ioniced = 0;
-			else if (ionicec == 3)
-				ioniced = 7;
-			ionicec <<= 13; /* class shift */
-		}
+	if ((tmp = getenv("SSD_IONICELEVEL"))) {
+		int n = sscanf(tmp, "%d:%d", &ionicec, &ioniced);
+		if (n != 1 && n != 2)
+			eerror("%s: invalid ionice level `%s' (SSD_IONICELEVEL)",
+			    applet, tmp);
+		if (ionicec == 0)
+			ioniced = 0;
+		else if (ionicec == 3)
+			ioniced = 7;
+		ionicec <<= 13; /* class shift */
+	}
 
 	/* Get our user name and initial dir */
 	p = getenv("USER");
@@ -766,6 +386,7 @@ int main(int argc, char **argv)
 		case 'c':  /* --chuid <username>|<uid> */
 			/* DEPRECATED */
 			ewarn("WARNING: -c/--chuid is deprecated and will be removed in the future, please use -u/--user instead");
+			/* falls through */
 		case 'u':  /* --user <username>|<uid> */
 		{
 			p = optarg;
@@ -854,7 +475,7 @@ int main(int argc, char **argv)
 			break;
 
 		case 's':  /* --signal <signal> */
-			sig = parse_signal(optarg);
+			sig = parse_signal(applet, optarg);
 			break;
 
 		case 't':  /* --test */
@@ -871,7 +492,7 @@ int main(int argc, char **argv)
 			startas = optarg;
 			break;
 		case 'w':
-			if (sscanf(optarg, "%d", &start_wait) != 1)
+			if (sscanf(optarg, "%u", &start_wait) != 1)
 				eerrorx("%s: `%s' not a number",
 				    applet, optarg);
 			break;
@@ -887,6 +508,14 @@ int main(int argc, char **argv)
 			redirect_stderr = optarg;
 			break;
 
+		case '3':   /* --stdout-logger "command to run for stdout logging" */
+			stdout_process = optarg;
+			break;
+
+		case '4':  /* --stderr-logger "command to run for stderr logging" */
+			stderr_process = optarg;
+			break;
+
 		case_RC_COMMON_GETOPT
 		}
 
@@ -918,7 +547,7 @@ int main(int argc, char **argv)
 	} else if (name) {
 		*--argv = name;
 		++argc;
-    } else if (exec) {
+	} else if (exec) {
 		*--argv = exec;
 		++argc;
 	};
@@ -938,6 +567,12 @@ int main(int argc, char **argv)
 		if (redirect_stdout || redirect_stderr)
 			eerrorx("%s: --stdout and --stderr are only relevant"
 			    " with --start", applet);
+		if (stdout_process || stderr_process)
+			eerrorx("%s: --stdout-logger and --stderr-logger are only relevant"
+			    " with --start", applet);
+		if (start_wait)
+			ewarn("using --wait with --stop has no effect,"
+			    " use --retry instead");
 	} else {
 		if (!exec)
 			eerrorx("%s: nothing to start", applet);
@@ -947,6 +582,15 @@ int main(int argc, char **argv)
 		if ((redirect_stdout || redirect_stderr) && !background)
 			eerrorx("%s: --stdout and --stderr are only relevant"
 			    " with --background", applet);
+		if ((stdout_process || stderr_process) && !background)
+			eerrorx("%s: --stdout-logger and --stderr-logger are only relevant"
+			    " with --background", applet);
+		if (redirect_stdout && stdout_process)
+			eerrorx("%s: do not use --stdout and --stdout-logger together",
+					applet);
+		if (redirect_stderr && stderr_process)
+			eerrorx("%s: do not use --stderr and --stderr-logger together",
+					applet);
 	}
 
 	/* Expand ~ */
@@ -962,26 +606,22 @@ int main(int argc, char **argv)
 		if (*exec == '/' || *exec == '.') {
 			/* Full or relative path */
 			if (ch_root)
-				snprintf(exec_file, sizeof(exec_file),
-				    "%s/%s", ch_root, exec);
+				xasprintf(&exec_file, "%s/%s", ch_root, exec);
 			else
-				snprintf(exec_file, sizeof(exec_file),
-				    "%s", exec);
+				xasprintf(&exec_file, "%s", exec);
 		} else {
 			/* Something in $PATH */
 			p = tmp = xstrdup(getenv("PATH"));
-			*exec_file = '\0';
+			exec_file = NULL;
 			while ((token = strsep(&p, ":"))) {
 				if (ch_root)
-					snprintf(exec_file, sizeof(exec_file),
-					    "%s/%s/%s",
-					    ch_root, token, exec);
+					xasprintf(&exec_file, "%s/%s/%s", ch_root, token, exec);
 				else
-					snprintf(exec_file, sizeof(exec_file),
-					    "%s/%s", token, exec);
-				if (exists(exec_file))
+					xasprintf(&exec_file, "%s/%s", token, exec);
+				if (exec_file && exists(exec_file))
 					break;
-				*exec_file = '\0';
+				free(exec_file);
+				exec_file = NULL;
 			}
 			free(tmp);
 		}
@@ -989,8 +629,12 @@ int main(int argc, char **argv)
 	if (start && !exists(exec_file)) {
 		eerror("%s: %s does not exist", applet,
 		    *exec_file ? exec_file : exec);
+		free(exec_file);
 		exit(EXIT_FAILURE);
 	}
+	if (start && retry)
+		ewarn("using --retry with --start has no effect,"
+		    " use --wait instead");
 
 	/* If we don't have a pidfile we should check if it's interpreted
 	 * or not. If it we, we need to pass the interpreter through
@@ -998,7 +642,10 @@ int main(int argc, char **argv)
 	if (interpreted && !pidfile) {
 		fp = fopen(exec_file, "r");
 		if (fp) {
-			p = fgets(line, sizeof(line), fp);
+			line = NULL;
+			if (getline(&line, &size, fp) == -1)
+				eerrorx("%s: %s", applet, strerror(errno));
+			p = line;
 			fclose(fp);
 			if (p != NULL && line[0] == '#' && line[1] == '!') {
 				p = line + 2;
@@ -1010,7 +657,8 @@ int main(int argc, char **argv)
 				if (p[len] == '\n')
 					p[len] = '\0';
 				token = strsep(&p, " ");
-				strncpy(exec_file, token, sizeof(exec_file));
+				free(exec_file);
+				xasprintf(&exec_file, "%s", token);
 				opt = 0;
 				for (nav = argv; *nav; nav++)
 					opt++;
@@ -1021,7 +669,7 @@ int main(int argc, char **argv)
 					nav[len++] = p;
 				for (i = 0; i < opt; i++)
 					nav[i + len] = argv[i];
-				nav[i + len] = '\0';
+				nav[i + len] = NULL;
 			}
 		}
 	}
@@ -1033,13 +681,20 @@ int main(int argc, char **argv)
 		if (!stop)
 			oknodo = true;
 		if (retry)
-			parse_schedule(retry, sig);
+			parse_schedule(applet, retry, sig);
 		else if (test || oknodo)
-			parse_schedule("0", sig);
+			parse_schedule(applet, "0", sig);
 		else
-			parse_schedule(NULL, sig);
-		i = run_stop_schedule(exec, (const char *const *)margv,
-		    pidfile, uid, test, progress);
+			parse_schedule(applet, NULL, sig);
+		if (pidfile) {
+			pid = get_pid(applet, pidfile);
+			if (pid == -1 && errno != ENOENT)
+				exit(EXIT_FAILURE);
+		} else {
+			pid = 0;
+		}
+		i = run_stop_schedule(applet, exec, (const char *const *)margv,
+		    pid, uid, test, progress, false);
 
 		if (i < 0)
 			/* We failed to stop something */
@@ -1061,14 +716,18 @@ int main(int argc, char **argv)
 	}
 
 	if (pidfile)
-		pid = get_pid(pidfile);
+		pid = get_pid(applet, pidfile);
 	else
 		pid = 0;
 
-	if (do_stop(exec, (const char * const *)margv, pid, uid,
-		0, test) > 0)
+	if (pid)
+		pids = rc_find_pids(NULL, NULL, 0, pid);
+	else
+		pids = rc_find_pids(exec, (const char * const *) argv, uid, 0);
+	if (pids)
 		eerrorx("%s: %s is already running", applet, exec);
 
+	free(pids);
 	if (test) {
 		if (rc_yesno(getenv("EINFO_QUIET")))
 			exit (EXIT_SUCCESS);
@@ -1254,6 +913,12 @@ int main(int argc, char **argv)
 				eerrorx("%s: unable to open the logfile"
 				    " for stdout `%s': %s",
 				    applet, redirect_stdout, strerror(errno));
+		}else if (stdout_process) {
+			stdout_fd = rc_pipe_command(stdout_process);
+			if (stdout_fd == -1)
+				eerrorx("%s: unable to open the logging process"
+				    " for stdout `%s': %s",
+				    applet, stdout_process, strerror(errno));
 		}
 		if (redirect_stderr) {
 			if ((stderr_fd = open(redirect_stderr,
@@ -1262,13 +927,21 @@ int main(int argc, char **argv)
 				eerrorx("%s: unable to open the logfile"
 				    " for stderr `%s': %s",
 				    applet, redirect_stderr, strerror(errno));
+		}else if (stderr_process) {
+			stderr_fd = rc_pipe_command(stderr_process);
+			if (stderr_fd == -1)
+				eerrorx("%s: unable to open the logging process"
+				    " for stderr `%s': %s",
+				    applet, stderr_process, strerror(errno));
 		}
 
 		if (background)
 			dup2(stdin_fd, STDIN_FILENO);
-		if (background || redirect_stdout || rc_yesno(getenv("EINFO_QUIET")))
+		if (background || redirect_stdout || stdout_process
+				|| rc_yesno(getenv("EINFO_QUIET")))
 			dup2(stdout_fd, STDOUT_FILENO);
-		if (background || redirect_stderr || rc_yesno(getenv("EINFO_QUIET")))
+		if (background || redirect_stderr || stderr_process
+				|| rc_yesno(getenv("EINFO_QUIET")))
 			dup2(stderr_fd, STDERR_FILENO);
 
 		for (i = getdtablesize() - 1; i >= 3; --i)
@@ -1323,9 +996,7 @@ int main(int argc, char **argv)
 		ts.tv_sec = start_wait / 1000;
 		ts.tv_nsec = (start_wait % 1000) * ONE_MS;
 		if (nanosleep(&ts, NULL) == -1) {
-			if (errno == EINTR)
-				eerror("%s: caught an interrupt", applet);
-			else {
+			if (errno != EINTR) {
 				eerror("%s: nanosleep: %s",
 				    applet, strerror(errno));
 				return 0;
@@ -1336,7 +1007,7 @@ int main(int argc, char **argv)
 				alive = true;
 		} else {
 			if (pidfile) {
-				pid = get_pid(pidfile);
+				pid = get_pid(applet, pidfile);
 				if (pid == -1) {
 					eerrorx("%s: did not "
 					    "create a valid"
@@ -1345,8 +1016,8 @@ int main(int argc, char **argv)
 				}
 			} else
 				pid = 0;
-			if (do_stop(exec, (const char *const *)margv,
-				pid, uid, 0, test) > 0)
+			if (do_stop(applet, exec, (const char *const *)margv,
+				pid, uid, 0, test, false) > 0)
 				alive = true;
 		}
 
